import os
import onnx
import ktc
import kneronnxopt
import cv2
import numpy as np  

def compute_sim(feature1, feature2):
    feat1 = feature1.ravel()
    feat2 = feature2.ravel()
    sim = np.dot(feat1, feat2) / (np.linalg.norm(feat1) * np.linalg.norm(feat2))
    return sim

def loadImage(filePath):
    image = cv2.imread(filePath)
    
    input_data = cv2.dnn.blobFromImage(image=image, size=(384,384), swapRB=True).astype(np.float32)
    
    return input_data

model_name = "liveness_efv2s_no_description"
weight_folder = "/media/workspace/weights/liveness/"
onnx_opt_path = "/media/workspace/weights/liveness/{}_optimized.onnx".format(model_name)
original_m = onnx.load("{}/{}.onnx".format(weight_folder,model_name))
model_input_name = "image"

model_index = 55663
model_version = "8b28"
model_platform =  "730"

quantize_folder = "/media/workspace/images/liveness_quant_1569" #量化要用的dataset

############################################################################################
#Step1 : 優化 Onnx 

optimized_m = kneronnxopt.optimize(original_m, skip_fuse_qkv=True)
onnx.save(optimized_m, onnx_opt_path)
############################################################################################
#Step2 : onnx to kneron(bie) 單獨檔案

km = ktc.ModelConfig(model_index, model_version, model_platform, onnx_model=optimized_m, debug=True)
# eval_result = km.evaluate()
# print(eval_result)

quantize_images = [loadImage(os.path.join(quantize_folder, fileName)) for fileName in os.listdir(quantize_folder)]
input_mapping = {model_input_name: quantize_images}

print(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>")
bie_path = km.analysis(input_mapping, output_dir=weight_folder,threads=4, optimize=0, mode=2, outlier_factor=2,compiler_tiling='deep_search', weight_bitwidth_mode="int8")
nef_path = ktc.compile([km], output_dir=weight_folder, dedicated_output_buffer=True, weight_compress=False)

############################################################################################
#Step3 : 測試 量化模型 bie 準確率 (單張影像)， onnx(float32) vs bie(int)
input_data = [loadImage("/media/workspace/images/liveness_quant_1569/mask_ROSE_YT_mask_full(58212)_frame_rotated_90_Mc_T_HS_g_E_7_38.mp4_15.png")]

# Inference with a bie file. `bie_path` is defined in section 1.5.1.
# kneron_fixed_output = ktc.kneron_inference(input_data, bie_file=bie_path, input_names=[model_input_name], platform=730)
kneron_fixed_output = ktc.kneron_inference(input_data, nef_file='weights/liveness/models_730.nef', input_names=[model_input_name], platform=730)

print("kneron fixed inference succefully")

kneron_f32_output = ktc.kneron_inference(input_data, onnx_file=onnx_opt_path, input_names=[model_input_name])

print("kneron onnx inference succefully")

print(compute_sim(np.asarray(kneron_fixed_output), np.asarray(kneron_f32_output)))

print("**********************************************")

for item1, item2 in zip(kneron_fixed_output[0][0], kneron_f32_output[0][0]):
    print(item1, item2)